#! /usr/bin/perl

# Stuart Levy, salevy@illinois.edu, 2011, for Laird Whitehill

foreach $_ (split(/:/, $ENV{'PATH'})) {
    push(@INC, $_), last if -f "$_/tfm.pl";
}
require 'tfm.pl';

sub Usage {
    print STDERR <<EOF;
Usage: perl $0 [-c cindex] [-l labellon] [-n npoints] infile.elements > outfile.speck
Reads a *comma*-separated input file of orbital elements, one per line, in the form
  name,a,e,i,node,peri[,cindex,labellon,npoints]
and writes a .speck file with an orbit curve, and optional label, for each object.
Options -c, -l, -n provide values for cindex, labellon, npoints if they're
omitted for a given object.  Defaults:  -c $cindex0   -l $labellon0   -n $npoints0
Fields mean:
    name -- name of object, e.g. 2003 QB1 or Sedna
    a  -- semimajor axis (in AU)    e  -- orbital eccentricity
    i  -- inclination            node  -- longitude of ascending node
    peri -- argument of perihelion
    cindex -- integer color index - sets color of orbit curve
    labellon -- ecliptic longitude of label, in any of several forms:
	none - no label
	N    - ecliptic longitude N in degrees
        p[+N] - longitude N relative to perihelion (just "p" puts label at peri point)
	Mc[+N] - ecliptic longitude N, plus M degrees per unit colorindex
	If, in elements file, labellon is missing or "-", the command-line -l value applies.
	Use "none" in elements file to suppress any label.
    npoints -- draw orbit curve using that number of points, equally spaced in longitude.
EOF
    exit(1);
}


$au2world = 4.8481368e-06;
$npoints0 = 360;
$cindex0 = 10;		# start after the planets
$labellon0 = "10c+30";	# Default label longitude: 30 degrees, plus 10 degrees per unit colorindex

&Usage if @ARGV == 0 && -t STDIN;

@Tz2w = @Tzg;

while(@ARGV && $ARGV[0] =~ /^-\w/) {
    $_ = shift(@ARGV);
    $cindex0 = shift(@ARGV), next if /^-c/;
    $labellon0 = shift(@ARGV), next if /^-l/;
    $npoints0 = shift(@ARGV), next if /^-n/;
    $au2world = 1, @Tz2w = &m3(&tfm(1)), next if /^-ecl/;
    $au2world = 1, @Tz2w = @Tze, next if /^-eq/;
    print STDERR "Unknown option $_\n" unless /-h/;
    &Usage;
}

while(<>) {
    $what = $_;
    s/#.*//;
    s/\s+$//;
    next unless /\d/;

    @F = split(/,/);
    if(@F == 1) {
	@F = split(' ', $_, );   # ok, it's blank-delimited
    }
    ($name, $a, $ecc, $incl, $node, $peri, $cindex, $labellon, $npoints) = @F;

    $npoints = $npoints0 unless $npoints > 0;
    $cindex = $cindex0++ unless $cindex > 0;

    my(@Tperi) = &tfm('z', $peri);	# longitude of perihelion
    my(@Tincl) = &tfm('x', $incl);
    my(@Tnode) = &tfm('z', $node);
    @Torbit = &tmul( @Tperi, @Tincl, @Tnode, &m4( @Tz2w ) );  # @Tzg is ecliptic-to-galactic transform, etc.

    print "\n";
    print "# $what\n";
    printf "mesh -s line -c %d { # %s\n%d 1\n", $cindex, $name, $npoints+1;
    print "#\tx y z # cindex pointno npoints name\n";

    my $k;
    for($k = 0; $k <= $npoints; $k++) {
	my $lon;
	if($ecc < 1) {
	    $lon = 360 * $k / $npoints;
	} else {
	    my($range) = &deg( &acos( -1 / $ecc ) );
	    $lon = 2 * $range*0.9995 * ($k / $npoints - 0.5);
	}
        printf "\t%.8g %.8g %.8g # %d %d %d %s\t\n", &lon2pgal($lon), $cindex, $k, $npoints, $name;
    }
    print "}\n";

    # labellon might have several forms:
    #  "-" or "" or "none" -- no label
    #  N  - ecliptic longitude in degrees
    #  p+N - orbital longitude relative to perihelion
    #  Mc+N - ecliptic longitude based on assigned color- M degrees per unit color index
    $labellon = $labellon0 if $labellon eq "" or $labellon eq "-";  # Apply default.  But preserve "none".
    $labelperi = ($labellon =~ s/\+?p//);
    if($labellon =~ s/([-+]?[.\d]+)c//) {
	$labelpercolor = $1;
    } else {
	$labelpercolor = undef;
    }
    if($labellon =~ /\d/ || $labelperi || defined($labelpercolor)) {
	# Put a label at that approx. longitude
	printf "%.8g %.8g %.8g text -size %g \"%s\"\n", &lon2pgal($labelpercolor*$cindex + $lon - ($labelperi ? 0 : $peri + $node)), abs($a * 0.5 * $au2world), $name;
    }
}

sub acos {
    my($c) = @_;
    return atan2( sqrt(1-$c**2), $c );
}

sub lon2pgal {
    # Convert longitude to a 3D point in galactic coordinates.  Uses globals $au2world, @Torbit, $a, $ecc
    my $theta = $_[0] * $pi / 180;
    my $r = $au2world * $a * (1 - $ecc*$ecc) / (1 + $ecc * cos($theta));
    my $x = $r * cos($theta);
    my $y = $r * sin($theta);
    my @pg = &v3mmul( $x,$y,0, @Torbit );
    return @pg;
}

__DATA__
Pluto,39.37,0.248803300000000,17.1671274745572,110.362856311729,114.358451817264,-,0
Mars,1.5236917,0.0934012067020093,1.85003831712244,49.5709808854595,-73.5293854267102,-,0
